@@ -1,5 +1,6 @@
 import re
 import urllib
+from urlparse import urlsplit
 import argparse
 import os
 import errno
@@ -24,7 +25,7 @@ class CourseraDownloader(object):
 
     DEFAULT_PARSER = "lxml"
 
-    def __init__(self,username,password,parser=DEFAULT_PARSER):
+    def __init__(self,username,password,proxy=None,parser=DEFAULT_PARSER):
         """Requires your coursera username and password. 
         You can also specify the parser to use (defaults to lxml), see http://www.crummy.com/software/BeautifulSoup/bs4/doc/#installing-a-parser
         """
@@ -33,6 +34,10 @@ class CourseraDownloader(object):
         self.parser = parser
 
         self.browser = Browser()
+        
+        if proxy:
+            self.browser.set_proxies({"http":proxy})
+
         self.browser.set_handle_robots(False)
 
     def login(self,course_name):
@@ -317,17 +322,30 @@ class CourseraDownloader(object):
 
     @staticmethod
     def getFileNameFromURL(url):
-        splits = url.split('/')
-        splits.reverse()
-        splits = urllib.unquote(splits[0])
-        #Seeing slash in the unquoted fragment
-        splits = splits.split('/')
-        fname = splits[len(splits) - 1]
+        # parse the url into its components
+        u = urlsplit(url)
+
+        # split the path into parts and unquote
+        parts = [urllib.unquote(x).strip() for x in u.path.split('/')]
+
+        # take the last component as filename
+        fname = parts[-1]
+        
+        # if empty, url ended with a trailing slash
+        # so join up the hostnam/path  and use that as a filename
+        if len(fname) < 1:
+           s = u.netloc + u.path[:-1]
+           fname = s.replace('/','_')
+        else:
+            # unquoting could have cuased slashes to appear again
+            # split and take the last element if so
+            fname = fname.split('/')[-1]
 
         # add an extension if none
         ext = os.path.splitext(fname)[1]
-        if not ext: fname += ".html"
-
+        if len(ext) < 1 or len(ext) > 5: fname += ".html"
+        
+        # remove any illegal chars and return
         return sanitiseFileName(fname)
 
 def sanitiseFileName(fileName):
@@ -427,6 +445,7 @@ def main():
     parser.add_argument("-d", dest='dest_dir', type=str, default=".", help='destination directory where everything will be saved')
     parser.add_argument("-q", dest='parser', type=str, default=CourseraDownloader.DEFAULT_PARSER,
                         help="the html parser to use, see http://www.crummy.com/software/BeautifulSoup/bs4/doc/#installing-a-parser")
+    parser.add_argument("-x", dest='proxy', type=str, default=None, help="proxy to use, e.g., foo.bar.com:3125")
     parser.add_argument('course_names', nargs="+", metavar='<course name>',
                         type=str, help='one or more course names (from the url)')
     args = parser.parse_args()
@@ -446,7 +465,7 @@ def main():
         args.password = getpass.getpass()
 
     # instantiate the downloader class
-    d = CourseraDownloader(args.username,args.password,parser=parser)
+    d = CourseraDownloader(args.username,args.password,proxy=args.proxy,parser=parser)
 
     # download the content
     for cn in args.course_names:
